package be.iminds.ilabt.jfed.espec.bundle;

import be.iminds.ilabt.jfed.git.GitFetcher;
import be.iminds.ilabt.jfed.git.GitFetcherCommand;
import be.iminds.ilabt.jfed.util.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import static be.iminds.ilabt.jfed.espec.bundle.ESpecBundle.ESPEC_DESCRIPTION_FILENAME;

/**
 * BundleFetcher convenience factory to fetch ESpec Bundles in different ways.
 */
public class BundleFetcher {
    private static final Logger LOG = LoggerFactory.getLogger(BundleFetcher.class);

    private BundleFetcher() { }

    @Nonnull public static ESpecBundle fetchDir(@Nonnull String dirname) throws ESpecBundle.ESpecBundleInitException {
        return fetchDir(new File(dirname), true);
    }
    @Nonnull public static ESpecBundle fetchDir(@Nonnull String dirname, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException {
        return fetchDir(new File(dirname), testExperimentSpecificationPresence);
    }
    @Nonnull public static ESpecBundle fetchDir(@Nonnull File dir) throws ESpecBundle.ESpecBundleInitException {
        return fetchDir(dir, true);
    }
    @Nonnull public static ESpecBundle fetchDir(@Nonnull File dir, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException {
        ESpecBundle res = new LocalDirESpecBundle(dir);
        if (testExperimentSpecificationPresence) {
            if (!res.isExperimentSpecificationPresent())
                throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
        }
        return res;
    }
    @Nonnull public static ESpecBundle fetchArchiveUrl(@Nonnull String url) throws ESpecBundle.ESpecBundleInitException, InterruptedException {
        return fetchArchiveUrl(url, true);
    }
    @Nonnull public static ESpecBundle fetchArchiveUrl(@Nonnull String url, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException, InterruptedException {
        try {
            return fetchArchiveUrl(new URL(url), testExperimentSpecificationPresence);
        } catch (MalformedURLException e) {
            throw new ESpecBundle.ESpecBundleInitException("Invalid URL", e);
        }
    }
    @Nonnull public static ESpecBundle fetchArchiveUrl(@Nonnull URL url) throws ESpecBundle.ESpecBundleInitException,InterruptedException {
        return fetchArchiveUrl(url, true);
    }
    @Nonnull public static ESpecBundle fetchArchiveUrl(@Nonnull URL url, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException,InterruptedException {
        try {
            File tmpFile = IOUtils.urlToTmpFile(url);
            LOG.debug("Downloaded ESpecBundle to "+tmpFile.getPath());
            ESpecBundle res = fetchArchiveFile(tmpFile);
            if (testExperimentSpecificationPresence) {
                if (!res.isExperimentSpecificationPresent())
                    throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
            }
            return res;
        } catch (IOException e) {
            throw new ESpecBundle.ESpecBundleInitException("Error downloading ESpecBundle", e);
        }
    }
    @Nonnull public static ESpecBundle fetchGitRepoDir(@Nonnull String gitFetcherSpec) throws ESpecBundle.ESpecBundleInitException,InterruptedException {
        return fetchGitRepoDir(gitFetcherSpec, true);
    }
    @Nonnull public static ESpecBundle fetchGitRepoDir(@Nonnull String gitFetcherSpec, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException,InterruptedException {
        try {
            GitFetcherCommand c = GitFetcherCommand.fromString(gitFetcherSpec)
                    .orElseThrow(() -> new IllegalArgumentException("gitFetcherSpec='"+gitFetcherSpec+"' is not a valid git source string"));
            File gitRepoDir = new GitFetcher().fetch(c);
            IOUtils.deleteDirAtExit(gitRepoDir);
            LOG.debug("Downloaded ESpec From repo to "+gitRepoDir);
            ESpecBundle res = fetchDir(gitRepoDir);
            if (testExperimentSpecificationPresence) {
                if (!res.isExperimentSpecificationPresent())
                    throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
            }
            return res;
        } catch (Exception e) {
            throw new ESpecBundle.ESpecBundleInitException(
                    "Error downloading ESpecBundle from git repo '"+gitFetcherSpec+"'", e);
        }
    }
    @Nonnull public static ESpecBundle fetchArchiveFile(@Nonnull String filename) throws ESpecBundle.ESpecBundleInitException {
        return fetchArchiveFile(filename, true);
    }
    @Nonnull public static ESpecBundle fetchArchiveFile(@Nonnull File filename) throws ESpecBundle.ESpecBundleInitException {
        return fetchArchiveFile(filename, true);
    }
    @Nonnull public static ESpecBundle fetchArchive(@Nonnull InputStream inputStream) throws ESpecBundle.ESpecBundleInitException {
        return fetchArchive(inputStream, true);
    }
    @Nonnull public static ESpecBundle fetchArchive(@Nonnull byte[] bytes) throws ESpecBundle.ESpecBundleInitException {
        return fetchArchive(bytes, true);
    }

    @Nonnull public static ESpecBundle fetchArchiveFile(@Nonnull String filename, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException {
        ESpecBundle res = fetchArchiveFile(new File(filename));
        if (testExperimentSpecificationPresence) {
            if (!res.isExperimentSpecificationPresent())
                throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
        }
        return res;
    }
    @Nonnull public static ESpecBundle fetchArchiveFile(@Nonnull File filename, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException {
        ESpecBundle res = new LocalArchiveESpecBundle(filename);
        if (testExperimentSpecificationPresence) {
            if (!res.isExperimentSpecificationPresent())
                throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
        }
        return res;
    }
    @Nonnull public static ESpecBundle fetchArchive(@Nonnull InputStream inputStream, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException {
        ESpecBundle res = new InMemoryArchiveESpecBundle(inputStream);
        if (testExperimentSpecificationPresence) {
            if (!res.isExperimentSpecificationPresent())
                throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
        }
        return res;
    }
    @Nonnull public static ESpecBundle fetchArchive(@Nonnull byte[] bytes, boolean testExperimentSpecificationPresence) throws ESpecBundle.ESpecBundleInitException {
        ESpecBundle res = new InMemoryArchiveESpecBundle(bytes);
        if (testExperimentSpecificationPresence) {
            if (!res.isExperimentSpecificationPresent())
                throw new RuntimeException(ESPEC_DESCRIPTION_FILENAME+" is not present in bundle");
        }
        return res;
    }
}
